package com.fjhx.iot.listener;

import cn.hutool.extra.spring.SpringUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.dynamic.datasource.toolkit.DynamicDataSourceContextHolder;
import com.fjhx.common.constant.SourceConstant;
import com.fjhx.iot.entity.tda.dto.MessageBody;
import com.fjhx.iot.entity.tda.po.TdaConfig;
import com.fjhx.iot.service.tda.TdaConfigService;
import com.fjhx.iot.utils.amqp.AmqpClient;
import com.fjhx.iot.utils.amqp.AmqpClientOptions;
import com.fjhx.iot.utils.amqp.AmqpConstants;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.framework.mybatis.holder.TenantHolder;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageConsumer;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * iot数据推送监听器
 *
 * @author zlj
 */
@Slf4j
@Component
public class DtaListener {

    private static final RedisCache redisCache = SpringUtil.getBean(RedisCache.class);

    private static final TdaConfigService tdaConfigService = SpringUtil.getBean(TdaConfigService.class);

    public static final Map<Long, MessageConsumer> listenerMap = new ConcurrentHashMap<>();

    @PostConstruct
    public void listener() {

        TenantHolder.setIgnore(true);
        DynamicDataSourceContextHolder.push(SourceConstant.IOT);

        List<TdaConfig> list = tdaConfigService.list();

        for (TdaConfig tdaConfig : list) {
            MessageConsumer consumer = getMessageConsumer(tdaConfig);
            listenerMap.put(tdaConfig.getId(), consumer);
        }

        DynamicDataSourceContextHolder.clear();
        TenantHolder.clear();

    }

    public static MessageConsumer getMessageConsumer(TdaConfig tdaConfig) {

        MessageConsumer consumer = null;
        try {
            AmqpClientOptions options = AmqpClientOptions.builder()
                    .host(tdaConfig.getEndPoint())
                    .port(AmqpConstants.PORT)
                    .accessKey(tdaConfig.getAccessKey())
                    .accessCode(tdaConfig.getAccessCode())
                    .queuePrefetch(1000)
                    .build();
            AmqpClient amqpClient = new AmqpClient(options);
            amqpClient.initialize();
            consumer = amqpClient.newConsumer(AmqpConstants.DEFAULT_QUEUE);
            // 此处进行消息处理。如果处理比较耗时，最好进行开启新的线程处理，否则可能造成心跳超时链接断开。
            consumer.setMessageListener(DtaListener::handleData);
            return consumer;
        } catch (Exception e) {
            try {
                if (consumer != null) {
                    consumer.close();
                }
            } catch (JMSException ex) {
                log.error("关闭失败", ex);
            }
            log.error("链接失败，tdaConfig:{}", tdaConfig.getId(), e);
            throw new ServiceException("连接iot失败");
        }
    }

    /**
     * 处理推送数据
     * <p>
     * remarks: 添加成功必须手动 message.acknowledge();
     * </p>
     *
     * @param message 推送消息
     */
    public static void handleData(Message message) {
        try {

            String messageBodyStr = message.getBody(String.class);
            MessageBody messageBody = JSONObject.parseObject(messageBodyStr, MessageBody.class);
            if (messageBody == null) {
                return;
            }

            // 事件类型
            String event = messageBody.getEvent();

            switch (event) {
                case "report":
                    reportHandel(messageBody);
                    break;
                default:
                    log.error("未知事件类型：{}\r\nmessageBody：{}", event, messageBodyStr);
                    return;
            }

            message.acknowledge();

        } catch (JMSException e) {
            e.printStackTrace();
        }

    }

    /**
     * 处理报告数据
     */
    private static void reportHandel(MessageBody messageBody) {

        MessageBody.NotifyData notifyData = messageBody.getNotifyData();

        if (notifyData == null) {
            log.error(" notifyData == null , messageBody = {}", messageBody);
            return;
        }

        MessageBody.Header header = notifyData.getHeader();
        if (header == null) {
            log.error(" header == null , messageBody = {}", messageBody);
            return;
        }

        MessageBody.Body body = notifyData.getBody();
        if (body == null) {
            log.error(" body == null , messageBody = {}", messageBody);
            return;
        }

        List<MessageBody.Service> services = body.getServices();
        if (services == null || services.size() == 0) {
            log.error(" services == null || services.size() == 0 , messageBody = {}", messageBody);
            return;
        }


        String productId = header.getProductId();

        String deviceId = header.getDeviceId();

        MessageBody.Service service = services.get(0);


        System.out.println(service);

    }


}
